@@ -15,16 +15,17 @@
 import json
 import time
 from ast import literal_eval
-from typing import Iterable, List, Optional, Tuple, cast
+from typing import Dict, List, Optional, Sequence, Tuple, Union, cast
 
 from requests import put
 from pytket.backends import Backend, CircuitStatus, ResultHandle, StatusEnum
 from pytket.backends.backend import KwargTypes
+from pytket.backends.backendinfo import BackendInfo, fully_connected_backendinfo
 from pytket.backends.backendresult import BackendResult
 from pytket.backends.resulthandle import _ResultIdTuple
 from pytket.backends.backend_exceptions import CircuitNotRunError
-from pytket.circuit import Circuit, OpType, Qubit  # type: ignore
-from pytket.device import Device  # type: ignore
+from pytket.circuit import Circuit, Node, OpType, Qubit  # type: ignore
+from pytket.extensions.aqt._metadata import __extension_version__
 
 
 from pytket.passes import (  # type: ignore
@@ -48,7 +49,6 @@ from pytket.predicates import (  # type: ignore
     NoSymbolsPredicate,
     Predicate,
 )
-from pytket.routing import FullyConnected  # type: ignore
 from pytket.utils import prepare_circuit
 from pytket.utils.outcomearray import OutcomeArray
 from .config import AQTConfig
@@ -63,12 +63,13 @@ _DEBUG_HANDLE_PREFIX = "_MACHINE_DEBUG_"
 
 # Hard-coded for now as there is no API to retrieve these.
 # All devices are fully connected.
-device_info = {
+_DEVICE_INFO = {
     AQT_DEVICE_QC: {"max_n_qubits": 4},
     AQT_DEVICE_SIM: {"max_n_qubits": 10},
     AQT_DEVICE_NOISY_SIM: {"max_n_qubits": 10},
 }
 
+_GATE_SET = {OpType.Rx, OpType.Ry, OpType.XXPhase, OpType.Measure, OpType.Barrier}
 
 AQTResult = Tuple[int, List[int]]  # (n_qubits, list of readouts)
 
@@ -127,19 +128,24 @@ class AQTBackend(Backend):
             raise AqtAuthenticationError()
 
         self._header = {"Ocp-Apim-Subscription-Key": access_token, "SDK": "pytket"}
-        if device_name in device_info:
-            self._max_n_qubits: Optional[int] = device_info[device_name]["max_n_qubits"]
-            self._device = FullyConnected(self._max_n_qubits)
-            self._qm = {Qubit(i): node for i, node in enumerate(self._device.nodes)}
-        else:
-            self._max_n_qubits = None
-            self._device = None
-            self._qm = {}
+        self._backend_info: Optional[BackendInfo] = None
+        self._qm: Dict[Qubit, Node] = {}
+        if device_name in _DEVICE_INFO:
+            self._backend_info = fully_connected_backendinfo(
+                type(self).__name__,
+                device_name,
+                __extension_version__,
+                _DEVICE_INFO[device_name]["max_n_qubits"],
+                _GATE_SET,
+            )
+            self._qm = {
+                Qubit(i): node for i, node in enumerate(self._backend_info.nodes)
+            }
         self._MACHINE_DEBUG = False
 
     @property
-    def device(self) -> Optional[Device]:
-        return self._device
+    def backend_info(self) -> Optional[BackendInfo]:
+        return self._backend_info
 
     @property
     def required_predicates(self) -> List[Predicate]:
@@ -148,12 +154,10 @@ class AQTBackend(Backend):
             NoFastFeedforwardPredicate(),
             NoMidMeasurePredicate(),
             NoSymbolsPredicate(),
-            GateSetPredicate(
-                {OpType.Rx, OpType.Ry, OpType.XXPhase, OpType.Measure, OpType.Barrier}
-            ),
+            GateSetPredicate(_GATE_SET),
         ]
-        if self._max_n_qubits is not None:
-            preds.append(MaxNQubitsPredicate(self._max_n_qubits))
+        if self._backend_info is not None:
+            preds.append(MaxNQubitsPredicate(self._backend_info.n_nodes))
         return preds
 
     def default_compilation_pass(self, optimisation_level: int = 1) -> BasePass:
@@ -203,8 +207,8 @@ class AQTBackend(Backend):
 
     def process_circuits(
         self,
-        circuits: Iterable[Circuit],
-        n_shots: Optional[int] = None,
+        circuits: Sequence[Circuit],
+        n_shots: Optional[Union[int, Sequence[int]]] = None,
         valid_check: bool = True,
         **kwargs: KwargTypes,
     ) -> List[ResultHandle]:
@@ -212,8 +216,22 @@ class AQTBackend(Backend):
         See :py:meth:`pytket.backends.Backend.process_circuits`.
         Supported kwargs: none.
         """
-        if n_shots is None or n_shots < 1:
-            raise ValueError("Parameter n_shots is required for this backend")
+        circuits = list(circuits)
+        n_shots_list: List[int] = []
+        if hasattr(n_shots, "__iter__"):
+            for n in cast(Sequence[Optional[int]], n_shots):
+                if n is None or n < 1:
+                    raise ValueError(
+                        "n_shots values are required for all circuits for this backend"
+                    )
+                n_shots_list.append(n)
+            if len(n_shots_list) != len(circuits):
+                raise ValueError("The length of n_shots and circuits must match")
+        else:
+            if n_shots is None:
+                raise ValueError("Parameter n_shots is required for this backend")
+            # convert n_shots to a list
+            n_shots_list = [cast(int, n_shots)] * len(circuits)
 
         if valid_check:
             self._check_all_circuits(circuits)
@@ -221,7 +239,7 @@ class AQTBackend(Backend):
         postprocess = kwargs.get("postprocess", False)
 
         handles = []
-        for i, c in enumerate(circuits):
+        for i, (c, n_shots) in enumerate(zip(circuits, n_shots_list)):
             if postprocess:
                 c0, ppcirc = prepare_circuit(c, allow_classical=False, xcirc=_xcirc)
                 ppcirc_rep = ppcirc.to_dict()
@@ -258,6 +276,14 @@ class AQTBackend(Backend):
             self._cache[handle] = dict()
         return handles
 
+    def _update_cache_result(
+        self, handle: ResultHandle, result_dict: Dict[str, BackendResult]
+    ) -> None:
+        if handle in self._cache:
+            self._cache[handle].update(result_dict)
+        else:
+            self._cache[handle] = result_dict
+
     def circuit_status(self, handle: ResultHandle) -> CircuitStatus:
         self._check_handle_type(handle)
         jobid = handle[0]
@@ -268,8 +294,8 @@ class AQTBackend(Backend):
         if self._MACHINE_DEBUG:
             n_qubits, n_shots = literal_eval(jobid[len(_DEBUG_HANDLE_PREFIX) :])  # type: ignore
             empty_ar = OutcomeArray.from_ints([0] * n_shots, n_qubits, big_endian=True)
-            self._cache[handle].update(
-                {"result": BackendResult(shots=empty_ar, ppcirc=ppcirc)}
+            self._update_cache_result(
+                handle, {"result": BackendResult(shots=empty_ar, ppcirc=ppcirc)}
             )
             statenum = StatusEnum.COMPLETED
         else:
@@ -283,8 +309,8 @@ class AQTBackend(Backend):
                     data["samples"], data["no_qubits"], big_endian=True
                 )
                 shots = shots.choose_indices(measure_permutations)
-                self._cache[handle].update(
-                    {"result": BackendResult(shots=shots, ppcirc=ppcirc)}
+                self._update_cache_result(
+                    handle, {"result": BackendResult(shots=shots, ppcirc=ppcirc)}
                 )
         return CircuitStatus(statenum, message)
 
